#ifdef HAS_KRB5

#include "krb5.h"

#include "auth.h"
#include "link.h"
#include "debug.h"
#include "domain_name_cache.h"

#include <string.h>
#include <stdlib.h>

#define SERVICE "host"
#define VERSION "dttools_protocol_1"

int krb5_net_read( krb5_context context, int fd, void *data, int length )
{
	struct link *link = (struct link*)fd;
	return link_read(link,data,length,time(0)+60);
}

int krb5_net_write( krb5_context context, int fd, void *data, int length )
{
	struct link *link = (struct link*)fd;
	return link_write(link,data,length,time(0)+60);
}

int auth_kerberos_assert( struct link *link, time_t stoptime )
{
	krb5_context context;
	krb5_ccache ccdef;
	krb5_principal client, server;
	krb5_data cksum;
	krb5_auth_context auth_context = 0;
	krb5_ap_rep_enc_part *rep_ret;
	krb5_error *err_ret;
	int success = 0;
	int port;

	char addr[LINK_ADDRESS_MAX];
	char dname[DOMAIN_NAME_MAX];

	debug("auth_kerberos: determining service name");

	link_address_remote(link,addr,&port);
	if(domain_name_cache_lookup_reverse(addr,dname)) {

		debug("auth_kerberos: name of %s is %s",addr,dname);
		cksum.data = dname;
		cksum.length = strlen(dname);

		debug("auth_kerberos: creating context");
		if(!krb5_init_context(&context)) {

			debug("auth_kerberos: opening credential cache");
			if(!krb5_cc_default(context,&ccdef)) {

				debug("auth_kerberos: loading my credentials");
				if(!krb5_cc_get_principal(context,ccdef,&client)) {

					char *name;
					krb5_unparse_name(context,client,&name);
					debug("auth_kerberos: I am %s",name);
					free(name);

					debug("auth_kerberos: building server principal");
					if(!krb5_sname_to_principal(context,dname,SERVICE,KRB5_NT_SRV_HST,&server)) {

						krb5_unparse_name(context,server,&name);
						debug("auth_kerberos: expecting server %s",name);
						free(name);
						debug("auth_kerberos: waiting for server");
						if(auth_barrier(link,"yes\n",stoptime)) {
							debug("auth_kerberos: authenticating with server");
							if(!krb5_sendauth(context,&auth_context,(krb5_pointer)&link,VERSION,client,server,AP_OPTS_MUTUAL_REQUIRED,&cksum,0,ccdef,&err_ret,&rep_ret,0)) {
								debug("auth_kerberos: credentials accepted!");
								krb5_free_ap_rep_enc_part(context,rep_ret);
								krb5_auth_con_free(context,auth_context);
								success = 1;
							} else {
								debug("auth_kerberos: couldn't authenticate to server");
							}
							krb5_free_principal(context,server);
						} else {
							debug("auth_kerberos: server couldn't load credentials");
						}
					} else {
						debug("auth_kerberos: couldn't build server principal");
						auth_barrier(link,"no\n",stoptime);
					}
					krb5_free_principal(context,client);
				} else {
					debug("auth_kerberos: couldn't retrieve my credentials");
					auth_barrier(link,"no\n",stoptime);
				}
				krb5_cc_close(context,ccdef);
 			} else {
				debug("auth_kerberos: couldn't open the credential cache");
				auth_barrier(link,"no\n",stoptime);
			}
	 		krb5_free_context(context);
		} else {
			debug("auth_kerberos: couldn't create a context");
			auth_barrier(link,"no\n",stoptime);
		}
	} else {
		debug("auth_kerberos: couldn't determine name of %s",addr);
		auth_barrier(link,"no\n",stoptime);
	}

	return success;
}

int auth_kerberos_accept( struct link *link, char **subject, time_t stoptime )
{
	krb5_context context;
	krb5_auth_context auth_context = NULL;
	krb5_ticket *ticket;
	krb5_principal principal;
	int success=0;

	debug("auth_kerberos: creating a context");
	if(!krb5_init_context(&context)) {

		debug("auth_kerberos: computing my service name");
		if(!krb5_sname_to_principal(context,NULL,SERVICE,KRB5_NT_SRV_HST,&principal)) {
			char *name;
			krb5_unparse_name(context,principal,&name);
			debug("auth_kerberos: I am %s",name);
			free(name);

			debug("auth_kerberos: waiting for client");
			if(auth_barrier(link,"yes\n",stoptime)) {

				debug("auth_kerberos: receiving client credentials");
				if(!krb5_recvauth(context,&auth_context,(krb5_pointer)&link,VERSION,principal,0,0,&ticket)) {

					debug("auth_kerberos: parsing client name");
					if(!krb5_unparse_name(context,ticket->enc_part2->client,subject)) {
						debug("auth_kerberos: client is %s\n",*subject);
						success = 1;
					} else {
						debug("auth_kerberos: couldn't parse client name");
					}
					krb5_auth_con_free(context,auth_context);
				} else {
					debug("auth_kerberos: couldn't receive client credentials");
				}
			} else {
				debug("auth_kerberos: client couldn't load credentials");
			}
		} else {
			debug("auth_kerberos: couldn't figure out my service name");
			auth_barrier(link,"no\n",stoptime);
		}
 		krb5_free_context(context);
	} else {
		debug("auth_kerberos: couldn't create kerberos context");
		auth_barrier(link,"no\n",stoptime);
	}

	return success;
}

int auth_kerberos_register()
{
	debug("auth_kerberos: registered");
	return auth_register("kerberos",auth_kerberos_assert,auth_kerberos_accept);
}

#else

#include "debug.h"

int auth_kerberos_register()
{
	debug("auth_kerberos: not compiled in");
	return 0;
}

#endif
