/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "datagram.h"
#include "debug.h"
#include "domain_name_cache.h"
#include "stringtools.h"
#include "rotating_file.h"
#include "username.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <signal.h>

#include "getopt.h"

static void stop_server( int sig )
{
	exit(0);
}

static void show_version( const char *cmd )
{
	printf("%s %s %s\n",cmd,__DATE__,__TIME__);
}

static void show_use( const char *cmd ) 
{
	fprintf(stderr,"Use: %s [options]\n",cmd);
	fprintf(stderr,"Where options are:\n");
	fprintf(stderr," -c        Send messages to console.\n");
	fprintf(stderr," -f <file> Send messages to this file.\n");
	fprintf(stderr," -s        Limit file to this size.\n");
	fprintf(stderr," -n        Do not perform reverse name lookups.\n");
	fprintf(stderr," -u <user> Run as this userid.\n");
	fprintf(stderr," -v        Show the version string\n");
	fprintf(stderr," -h        Show this help screen.\n");
}

int main( int argc, char *argv[] )
{
	char buffer[DATAGRAM_PAYLOAD_MAX];
	char addr[DATAGRAM_ADDRESS_MAX];
	char name[DOMAIN_NAME_MAX];
	char  c;

	char *username=0;
	char *debug_file=0;
	int debug_file_size = 65536;

	struct datagram *d;

	int port = DEBUG_PORT;

	int do_name = 1;
	int do_file = 0;
	int do_console = 0;

	while((c=getopt(argc,argv,"cf:s:nu:vh"))!=(char)-1) {
		switch(c) {
			case 'c':
				do_console = 1;
				break;
			case 'f':
				do_file = 1;
				debug_file = optarg;
				break;
			case 's':
				debug_file_size = atoi(optarg);
				break;
			case 'u':
				username = optarg;
				break;
			case 'n':
				do_name = 0;
				break;
			case 'v':
				show_version(argv[0]);
				return 1;
			case 'h':
			default:
				show_use(argv[0]);
				return 1;
				break;
		}
	}

	signal(SIGPIPE,SIG_IGN);
	signal(SIGHUP,SIG_IGN);
	signal(SIGQUIT,stop_server);
	signal(SIGTERM,stop_server);

	if(username) {
		if(!username_set(username)) {
			fatal("Couldn't switch to user %s!",username);
		}
	} else {
		if(username_is_super()) {
			fatal("I refuse to run as the superuser.  Please use the -u option.");
		}
	}

	if(!do_file && !do_console) {
		fatal("You must give at least one of -c and -f.");
	}

	d = datagram_create( port );
	if(!d) {
	       fatal("couldn't listen on port %d: %s",port,strerror(errno));
	}

	sprintf(buffer,"%s started.",argv[0]);
	datagram_send(d,buffer,strlen(buffer),"0.0.0.0",port);

	while(1) {
		char message[DATAGRAM_PAYLOAD_MAX*2];
		int result;
		int length;
		time_t t;

		result = datagram_recv(d,buffer,sizeof(buffer),addr,&port,60);
		if(result<0) continue;

		buffer[result] = 0;
		string_chomp(buffer);

		message[0] = 0;

		time(&t);
		strftime(message,sizeof(message),"%d-%b-%Y %H:%M:%S ", localtime(&t));

		result = domain_name_cache_lookup_reverse(addr,name);
		if(!result) strcpy(name,addr);

		sprintf(&message[strlen(message)],"%s %s\n",name,buffer);
		length = strlen(message);

		if( do_file ) {
			rotating_file_write( debug_file, debug_file_size, message, length );
		}

		if( do_console ) {
			fwrite(message,length,1,stderr);
		}
	}
}
