
#include "shell.h"

#include "hash_table.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef HAS_LIBREADLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

#define SHELL_LINE_MAX 1024

struct shell {
	char *prompt;
	shell_command_t default_command;
	struct hash_table *table;
};

struct shell * shell_create( char *prompt )
{
	struct shell *s;

	s = malloc(sizeof(*s));
	if(s) {
		s->prompt = strdup(prompt);
		if(s->prompt) {
			s->table = hash_table_create( 127, hash_string );
			if(s->table) {
				s->default_command = 0;
				return s;
			}
			free(s->prompt);
		}
		free(s);
	}
	return 0;
}

void shell_delete( struct shell *s )
{
	if(s) {
		if(s->prompt) free(s->prompt);
		if(s->table) free(s->table);
		free(s);
	}
}

int shell_register( struct shell *s, const char *name, shell_command_t cmd )
{
	return hash_table_insert( s->table, name, cmd, 0 );
}

int shell_register_default( struct shell *s, shell_command_t cmd )
{
	s->default_command = cmd;
	return 1;
}

#ifndef HAS_LIBREADLINE

static char * readline( char *prompt )
{
	char line[SHELL_LINE_MAX];

	printf("%s",prompt);
	fflush(stdout);

	if(fgets(line,SHELL_LINE_MAX,stdin)) {
		line[strlen(line)-1] = 0;
		return strdup(line);
	} else {
		return 0;
	}
}

static void add_history( const char *line )
{
}

#endif

static char ** parseline( char *line, int *argc )
{
	int maxargs=10;
	char **argv;

	*argc=0;

	argv = (char**) malloc(maxargs*sizeof(char*));
	if(!argv) return 0;

	for( (*argc)=0; (*argc)<maxargs; (*argc)++ ) {
		argv[*argc] = strtok(line," \t");
		if(line) line=0;
		if(!argv[*argc]) break;
	}

	if( (*argc)==maxargs ) {
		argv[(*argc)-1] = 0;
	}

	return argv;
}

void shell_run( struct shell *s )
{
	shell_command_t cmd;
	char *line=0;
	char **argv=0;
	int argc;

	while(1) {
		if(line) {
			free(line);
			line = 0;
		}

		if(argv) {
			free(argv);
			argv = 0;
		}

	        line = readline( s->prompt );
		if(!line) break;

		add_history(line);

		argv = parseline(line,&argc);
		if(!argv) continue;
		if(!argv[0]) continue;
		if(!*argv[0]) continue;

		argc--;

		cmd = hash_table_lookup(s->table,argv[0]);
		if(!cmd) cmd = s->default_command;
		
		if(!cmd) {
			fprintf(stderr,"%s: unknown command (type 'help' for help)",argv[0]);
		} else {
			if(!cmd(s,argc,argv)) {
				break;
			}
		}
	}

	printf("\n");
}

